#define _FILE_OFFSET_BITS 64

#include <getopt.h>
#include <stdio.h>
#include <unistd.h>

void help()
{
  printf("truncate - truncate files to a specified length\n");
  printf("\n");
  printf("if the length is smaller than the current filelength, data is lost.\n");
  printf("if the length if greater, the file is padded with 0.\n");
  printf("\n");
  printf("usage: truncate [-v] -s <size> <filename>\n");
  printf("size argument: an integer optionally followed by one the the letters:\n");
  printf("bBkKmMgG.\n");
  printf("b or B: size in bytes\n");
  printf("k: size in 1000 bytes\n");
  printf("K: size in 1024 bytes\n");
  printf("m: size in 10^6 bytes\n");
  printf("M: size in 1024^2 bytes\n");
  printf("g: size in 10^9 bytes\n");
  printf("G: size in 1024^3 bytes\n");
  exit(0);
}

int main(int argc, char **argv)
{
  unsigned long long size=0;
  int c, force=0;
  char *filename;

  while((c=getopt(argc, argv, "vhfs:")) >= 0)
    switch(c)
      {
      case '?':
      case 'h':
	help();

      case 'v':
	printf("truncate version 2005-12-02\n");
	exit(1);

      case 'f': force=1; break;
      case 's':
	{
	  int i=0;
	  char *s=optarg;
	  size=atoll(s);
	  while(s[i]!=0)
	    {
	      switch(s[i])
		{
		case 'B':
		case 'b': break;

		case 'K': size*=1024; break;
		case 'k': size*=1000; break;

		case 'M': size*=1024*1024; break;
		case 'm': size*=1000*1000; break;

		case 'G': size*=1024*1024*1024; break;
		case 'g': size*=1000*1000*1000; break;

		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
		  break;

		default:
		  fprintf(stderr, "unknown size option for -s argument: %c\n", s[i]);
		  exit(1);
		}
	      ++i;
	    }
	}
    }

  if(optind==argc)
    {
      fprintf(stderr, "no filename given\n");
      exit(1);
    }

  filename=argv[optind];

  if(!force)
    {
      fprintf(stderr, "I will truncate %s to %llu bytes\nto really truncate use the -f (force) switch.\n", filename, size);
      exit(0);
    }

  if(truncate(filename, size) < 0)
    {
      perror("could not truncate");
      exit(1);
    }

  return 0;
}
